#!/usr/bin/env python
# encoding: utf-8
"""
@author: 风起
@contact: onlyzaliks@gmail.com
@File: createmap.py
@Time: 2021/12/6 8:55
"""


def create_data_map(data, data_map_name):
    html = str(data_map_name)
    body = '{width: 100%;height: 100%;overflow: hidden;margin:0;font-family:"微软雅黑";}'
    f = open(html, 'w', encoding='utf-8')
    message = """
    <!DOCTYPE html> 
    <html> 
        <head> 
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <meta name="viewport" content="initial-scale=1.0, user-scalable=no" />
        <link rel="shortcut icon" href="https://gitee.com/gui-yunyou/kunyu_ico/raw/master/favicon.ico" type="image/x-icon">
        <style type="text/css"> 
            body, html,#allmap%s
            .BMap_cpyCtrl {
                display: none !important;
            }
            .anchorBL {
                display: none !important;
            }
        </style> 
        <script type="text/javascript" 
            src="http://api.map.baidu.com/api?v=2.0&ak=WjqIEPES5kODPcbszyt29MhBX4FC2xa0">
        </script> 
        <title>Kunyu（坤舆）资产分布图</title> 
        </head> 
        <body> 
        <div id="allmap"></div> 
        </body> 
    </html> 
    
    <script type="text/javascript">
        var points = %s;
        function addMarker(points) {
            for(var i=0, pointsLen = points.length; i<pointsLen; i++) {
                var point = new BMap.Point(points[i].lng, points[i].lat);
                var marker = new BMap.Marker(point);
                map.addOverlay(marker);
                var thePoint = points[i];
                //Listen for mouse click events
                marker.addEventListener("click",
                function() {
                    showInfo(this,thePoint);
                });
            }
        }
        
        //Set the description of the marker
        function showInfo(thisMarker,point) {
            var sContent = 
                '<ul style="margin:0 0 5px 0;padding:0.2em 0">'  
                +'<li style="line-height: 35px;font-family:"楷体";font-size: 30px;">'  
                +'<span style="width: 50px;display: inline-block;">IP:</span>' + point.ip + '</li>'  
                +'<li style="line-height: 35px;font-family:"楷体";font-size: 30px;">'  
                +'<span style="width: 50px;display: inline-block;"> 纬度:</span>' +point.lat +
                '<br><span style="width: 50px;display: inline-block;"> 经度:</span>' + point.lng +'</li>'  
                +'</ul>';
            var infoWindow = new BMap.InfoWindow(sContent);
            
            thisMarker.openInfoWindow(infoWindow);
            }
            
            //Generate the map and set
            var map = new BMap.Map("allmap");
            map.centerAndZoom(new BMap.Point(116.418261,39.921984), 5);
            map.addControl(new BMap.MapTypeControl({mapTypes: [BMAP_NORMAL_MAP,BMAP_SATELLITE_MAP,BMAP_HYBRID_MAP]}));
            map.addControl(new BMap.ScaleControl({anchor: BMAP_ANCHOR_TOP_LEFT}));
            map.addControl(new BMap.OverviewMapControl());
            map.addControl(new BMap.OverviewMapControl({isOpen:true, anchor: BMAP_ANCHOR_BOTTOM_RIGHT}))
            map.addControl(new BMap.NavigationControl());
            map.enableScrollWheelZoom(true);     
            addMarker(points);
    </script>
    
    """ % (body, data)
    f.write(message)
    f.close()



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: encode.py
@Time: 2021/6/15 15:40
'''

import re
import os
import base64
import codecs
import hashlib
import socket

import ssl
import mmh3
import requests
from tld import get_fld

from kunyu.utils.log import logger
from kunyu.config.setting import IP_ADDRESS_REGEX, HTTP_CHECK_REGEX


class EncodeHash:
    """"
        According to different search engines,
        Select the appropriate ICO icon encryption method.
        ZoomEye, for example, supports both MMH3 and MD5,But FoFa only supports MMH3.
        Through the ICO icon search Related assets,It's very always efficient.
        Security researchers can also modify code files as needed.
    """

    def __init__(self, func):
        self.filename = None
        self.status = False
        self.func = func

    def __call__(self, *args, **kwargs):
        self.filename, self.status = self.func(*args, **kwargs)
        icohash = EncodeHash.http_encode(self) if self.check_http() else self.file_encode()
        return icohash if icohash is not None else logger.warning("The hash was not successfully computed")

    def check_http(self):
        # Determine if it is valid URL.
        if re.search(IP_ADDRESS_REGEX, self.filename):
            return True

        if re.findall(HTTP_CHECK_REGEX, self.filename):
            if get_fld(self.filename):
                return True
            return False

    # Get File favicon Images Encode Hash
    def file_encode(self):
        # MD5 encrypted files.
        if os.path.isfile(self.filename):
            fp = open(self.filename, 'rb')
            contents = fp.read()
            fp.close()
        else:
            return logger.warning("No Such File or URL!")

        # Select an encryption mode
        return hashlib.md5(contents).hexdigest() if self.status else mmh3.hash(
            codecs.lookup('base64').encode(contents)[0])

    # Get HTTP favicon Images Encode Hash
    def http_encode(self):
        try:
            # Select an encryption mode
            if self.status:
                return hashlib.md5(requests.get(self.filename, timeout=0.5).content).hexdigest()
            else:
                return mmh3.hash(codecs.lookup('base64').encode(requests.get(self.filename, timeout=0.5).content)[0])
        except:
            return


# Calculate md5 Hash
@EncodeHash
def encode_md5(filename):
    return filename, True


# Calculate mmh3 Hash
@EncodeHash
def encode_mmh3(filename):
    return filename, False


# encode hex
def encode_hex(string):
    return int(string, 16)


# encode base64
def encode_base64(string):
    bytes_str = string.encode("utf-8")
    return str(base64.b64encode(bytes_str).decode("utf-8"))


# Cert series number calculate Hash
def cert_encode(hostname):
    try:
        c = ssl.create_default_context()
        host = re.sub(HTTP_CHECK_REGEX, '', hostname)
        s = c.wrap_socket(socket.socket(), server_hostname=host)
        s.settimeout(5)
        s.connect((host, 443))
        # Return Hexadecimal code
        return encode_hex(s.getpeercert()["serialNumber"])

    except Exception:
        return logger.warning("Please confirm that the target uses HTTPS or is accessible")






#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: crash.py
@Time: 2021/9/2 17:54
'''
import re
import json
import random
import sys

import grequests
import requests

from kunyu.core import conf
from rich.console import Console
from kunyu.utils.log import logger
from kunyu.utils.convert import convert
from kunyu.lib.batchfile import get_domain_file, get_file
from kunyu.config.setting import UA, DOMAIN_SEARCH_API, DOMAIN_CHECK_REGEX, IP_ADDRESS_REGEX

console = Console(color_system="auto", record=True)
ZOOMEYE_KEY = conf.get("zoomeye", "apikey")
ZOOMEYE_TOKEN = conf.get("login", "token")
SERVERLESS_API = conf.get("Serverapi", "serverless")


class HostScan:
    def __init__(self):
        self.headers = {
            "User-Agent": random.choice(UA)
        }
        self.params = {"type": 1}
        self.IP_STATUS = 0
        self.__get_login()

    # Check whether the HTTP request returns an error
    def __check_error(self, resp):
        if resp.get("error"):
            raise requests.HTTPError(resp.get("message"))

    # Dynamically obtain user credentials according to the initialized configuration file
    def __get_login(self):
        if ZOOMEYE_KEY == "None":
            self.headers["Authorization"] = "JWT %s" % ZOOMEYE_TOKEN
        else:
            self.headers["API-KEY"] = ZOOMEYE_KEY

    # Get ZoomEye Doamin List
    def __get_zoomeye_domain(self):
        """
            Return to the result set of the reverse search through the zoomeye domain name library
            First get the number of result sets in the first HTTP request
            Perform calculations based on the number of result sets to dynamically obtain all results
        """
        domain_list = []
        try:
            def request_get():
                resp = requests.get(
                    DOMAIN_SEARCH_API,
                    data=self.params,
                    headers=self.headers,
                    verify=False
                )
                self.__check_error(json.loads(resp.text))
                return resp.json()

            result = request_get()
            # Dynamically calculate the number of pages that need to be queried to obtain all result sets
            count = int(result["total"] / 30)
            page = count if (result["total"] % 30) == 0 else count + 1
            # Get ZoomEye Domain result
            for i in range(page):
                self.params["page"] = str(i + 1)
                result = request_get()
                for num in range(len(result["list"])):
                    data = convert(result["list"][num])
                    domain_list.append(data.name)
            # Remove duplicate domain names
            domain_list = list(set(domain_list))
            console.log("Host Header Scan Domain Total: ", len(domain_list), style="green")
            return domain_list

        except requests.HTTPError as err:
            logger.error(err)

    def __is_valid_domain(self, search):
        """
            Return whether or not given value is a valid domain.
            If the value is valid domain name this function returns ``True``, otherwise False
            :param search: domain string to validate
        """
        pattern = re.compile(DOMAIN_CHECK_REGEX)
        return True if pattern.match(search) else False

    def __is_valid_ip(self, ip):
        """
            Return whether or not given value is a valid ip address.
            If the value is valid ip address this function returns ``True``, otherwise False
            :param ip: ip string to validate
        """
        pattern = re.compile(IP_ADDRESS_REGEX)
        return True if pattern.match(ip) else False

    def _get_file(self, search):
        """
            Get the array of domain names required for HOST collision
            If it is the main domain name, check it through the zoomeye interface
            Otherwise, read the result through the file path
            :param search: Enter the main domain name or domain name file path
        """
        domain_list = []
        # Check the logic of obtaining the domain name result
        if self.__is_valid_domain(search):
            domain_list = self.__get_zoomeye_domain()
        else:
            for domain in get_domain_file(search):
                domain_list.append(str(domain))
            # Return to the domain name combing in the file
            console.log("Host Header Scan Domain Total: ", len(domain_list), style="green")
        return domain_list

    def _get_ip_file(self, ip):
        """
            Get the array of ip address required for HOST collision
            If it is IP, directly HOST collision
            Otherwise, read the file to obtain the IP address
            :param ip: Enter the main ip address or ip address file path
        """
        ip_list = []
        if self.__is_valid_ip(ip):
            ip_list.append(ip)
        else:
            for ip_address in get_file(ip):
                ip_list.append(ip_address)
        return ip_list

    def host_scan(self, search, ip):
        """
            Obtain hidden assets through HOST collision
            :param search: Ways to obtain domain names
            :param ip: IP address to be collided
        """
        resp, crash_list = [], []
        tmp_list = []
        status = False
        if SERVERLESS_API != "None": status = True
        # http and https protocol collision
        protocol = ['http://{}/', 'https://{}/']
        self.params["q"] = search
        domain_list = self._get_file(search)
        url = self._get_ip_file(ip)
        for server in protocol:
            for ip_address in url:
                urls = server.format(ip_address)
                resp_url = SERVERLESS_API if status else urls
                for domain in domain_list:
                    headers = {
                        'User-Agent': random.choice(UA),
                        'ip': urls,
                    }
                    # The cloud function sets header hosts
                    if status:headers["hosts"] = domain.strip()
                    else:headers["host"] = domain.strip()
                    # Concurrent requests through the encapsulated coroutine module
                    resp.append(grequests.get(
                        resp_url, headers=headers, timeout=5, verify=False)
                    )

        res_list = grequests.map(resp)
        for res in res_list:
            try:
                # Determine the HTTP status code that needs to be obtained
                if res.status_code == 200 or res.status_code == 302 or res.status_code == 301:
                    if res.text != "false":
                        res.encoding = 'gbk2312'
                        # Get the title of the returned result
                        title = re.findall('<title>(.+)</title>', res.text)
                        tmp_list = [res.request.headers['ip'],title[0]]
                        if status:
                            tmp_list.insert(1,res.request.headers['hosts'])
                            # Unicode decoding of the information returned by cloud functions
                            tmp_list[2] = (title[0].encode('utf8').decode('unicode_escape'))
                        else:tmp_list.insert(1,res.request.headers['host'])
                        crash_list.append(tmp_list)
            except Exception:
                continue

        return crash_list




#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: setting.py
@Time: 2021/6/15 16:50
This code file is used to set global parameters
'''

import os
import platform

PLATFORM = platform.system()

# ZoomEye API
USER_INFO_API = "https://api.zoomeye.org/resources-info"
HOST_SEARCH_API = "https://api.zoomeye.org/host/search"
WEB_SEARCH_API = "https://api.zoomeye.org/web/search"
BOTH_SEARCH_API = "https://api.zoomeye.org/both/search"
DOMAIN_SEARCH_API = "https://api.zoomeye.org/domain/search"

# Setting ZoomEye Query Fields
ZOOMEYE_FIELDS_HOST = ["ID", "IP", "Port", "Protocol", "Service", "ISP", "Country", "City", "Title", "Time"]
ZOOMEYE_FIELDS_WEB = ["ID", "IP", "URL", "Title", "OS", "WebApp", "DB", "Language", "Server", "Time"]
ZOOMEYE_FIELDS_INFO = ["Name","Plan", "Search", "Expired_AT"]
ZOOMEYE_FIELDS_DOMAIN = ["ID", "Domain", "IP", "TimeStamp"]
COMMAND_INFO = ["Options", "Value", "Info"]
SEMSITIVE_INFO = ["ID", "SEMSITIVE"]
HOST_SCAN_INFO = ["IP", "Domain", "Title"]

# Set executable system commands
OS_SYSTEM = []
if PLATFORM == "Windows":
        OS_SYSTEM = ["ipconfig", "dir", "whoami", "ping", "telnet", "cd", "findstr", "chdir","find", "mysql", "type", "curl", "netstat", "tasklist", "taskkill", "tracert", "del", "ver", "nmap"]
else:
        OS_SYSTEM = ["ifconfig", "ls", "cat", "pwd", "whoami", "ping", "find", "grep", "telnet", "mysql", "cd", "vi", "more", "less", "curl", "ps", "netstat", "rm", "touch", "mkdir", "uname", "nmap"]

# Kunyu OUTPUT File Path
OUTPUT_PATH = os.path.expanduser('~/kunyu/output/')

# REGEX rule
IP_ADDRESS_REGEX = r"(([01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])\.){3}([01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])"
HTTP_CHECK_REGEX = r"^https?:/{2}\w.+$"
DOMAIN_CHECK_REGEX = r'^(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|'\
    r'([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|'\
    r'([a-zA-Z0-9][-_.a-zA-Z0-9]{0,61}[a-zA-Z0-9])).'\
    r'([a-zA-Z]{2,13}|[a-zA-Z0-9-]{2,30}.[a-zA-Z]{2,3})$'

UA = [
        "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50",
        "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-us) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50",
        "Mozilla/5.0 (Windows NT 10.0; WOW64; rv:38.0) Gecko/20100101 Firefox/38.0",
        "Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; InfoPath.3; rv:11.0) like Gecko",
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/20100101 Firefox/4.0.1",
        "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/20100101 Firefox/4.0.1",
        "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; en) Presto/2.8.131 Version/11.11",
        "Opera/9.80 (Windows NT 6.1; U; en) Presto/2.8.131 Version/11.11",
        "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5",
        "Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5",
        "Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5",
        "Mozilla/5.0 (Linux; U; Android 2.3.7; en-us; Nexus One Build/FRF91) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
        "MQQBrowser/26 Mozilla/5.0 (Linux; U; Android 2.3.7; zh-cn; MB200 Build/GRJ22; CyanogenMod-7) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1",
        "Opera/9.80 (Android 2.3.4; Linux; Opera Mobi/build-1107180945; U; en-GB) Presto/2.8.149 Version/11.10",
        "Mozilla/5.0 (Linux; U; Android 3.0; en-us; Xoom Build/HRI39) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13",
        "Mozilla/5.0 (BlackBerry; U; BlackBerry 9800; en) AppleWebKit/534.1+ (KHTML, like Gecko) Version/6.0.0.337 Mobile Safari/534.1+",
        "Mozilla/5.0 (hp-tablet; Linux; hpwOS/3.0.0; U; en-US) AppleWebKit/534.6 (KHTML, like Gecko) wOSBrowser/233.70 Safari/534.6 TouchPad/1.0",
        "Mozilla/5.0 (SymbianOS/9.4; Series60/5.0 NokiaN97-1/20.0.019; Profile/MIDP-2.1 Configuration/CLDC-1.1) AppleWebKit/525 (KHTML, like Gecko) BrowserNG/7.1.18124",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; HTC; Titan)",
        "Mozilla/4.0 (compatible; MSIE 6.0; ) Opera/UCWEB7.0.2.37/28/999",
        "Mozilla/6.0 (iPhone; CPU iPhone OS 8_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/8.0 Mobile/10A5376e Safari/8536.25",
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.100 Safari/537.36",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_3_1 like Mac OS X; zh-CN) AppleWebKit/537.51.1 (KHTML, like Gecko) Mobile/17D50 UCBrowser/12.8.2.1268 Mobile AliApp(TUnionSDK/0.1.20.3)",
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon 2.0)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; The World)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SE 2.X MetaSr 1.0; SE 2.X MetaSr 1.0; .NET CLR 2.0.50727; SE 2.X MetaSr 1.0)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; 360SE)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Avant Browser)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)",
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.116 Safari/537.36",
        "Mozilla/5.0 (Linux; Android 8.1.0; OPPO R11t Build/OPM1.171019.011; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/76.0.3809.89 Mobile Safari/537.36 T7/11.19 SP-engine/2.15.0 baiduboxapp/11.19.5.10 (Baidu; P1 8.1.0)",
        "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_3_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 SP-engine/2.14.0 main%2F1.0 baiduboxapp/11.18.0.16 (Baidu; P2 13.3.1) NABar/0.0",
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.140 Safari/537.36 Edge/17.17134",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 12_4_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 MicroMessenger/7.0.10(0x17000a21) NetType/4G Language/zh_CN",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.108 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.106 Safari/537.36"
]



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: convert.py
@Time: 2021/7/3 0:47
'''


class Dict(dict):
    __setattr__ = dict.__setitem__
    __getattr__ = dict.__getitem__


# Dict conversion object
def convert(dictObj):
    if not isinstance(dictObj, dict):
        return dictObj
    json = Dict()
    for k, v in dictObj.items():
        json[k] = convert(v)
    return json



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: log.py
@Time: 2021/6/15 15:54
'''

import logging
import colorlog  # 控制台日志输入颜色

logger = logging.getLogger("kunyu-log")
logger_console = logging.getLogger("kunyu-console")


def console():
    handler = logging.StreamHandler()
    log_colors_config = {
        'DEBUG': 'cyan',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'red',
    }
    formatter = colorlog.ColoredFormatter(
        '%(log_color)s[%(asctime)s] [%(filename)s:%(lineno)d] [%(levelname)s]- %(message)s',
        log_colors=log_colors_config,
        datefmt="%H:%M:%S",
    )
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)


def log_console():
    handler = colorlog.StreamHandler()
    handler.setFormatter(
        colorlog.ColoredFormatter(
            fmt='%(log_color)s%(message)s',
            datefmt="%H:%M:%S",
            log_colors={
                'DEBUG': 'cyan',
                'INFO': 'white',
                'WARNING': 'bold_yellow',
                'ERROR': 'red',
                'CRITICAL': 'red',
            },
        )
    )
    logger_console.addHandler(handler)
    logger_console.setLevel(logging.DEBUG)


console()
log_console()



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: __version__.py
@Time: 2021/6/15 21:11
'''

import sys
import platform

__title__ = 'kunyu'
__license__ = 'GPL-2.0'
__python_version__ = sys.version.split()[0]
__platform__ = platform.platform()
__url__ = "https://github.com/knownsec/Kunyu"
__version__ = '1.6.3'
__author__ = '风起'
__Team__ = 'KnownSec 404 Team'
__author_email__ = 'onlyzaliks@gmail.com'
__copyright__ = 'Copyright (C) 2021 风起. All Rights Reserved'
__introduction__ = """

██╗  ██╗  ██╗   ██╗  ███╗   ██╗  ██╗   ██╗  ██╗   ██╗
██║ ██╔╝  ██║   ██║  ████╗  ██║  ╚██╗ ██╔╝  ██║   ██║
█████╔╝   ██║   ██║  ██╔██╗ ██║   ╚████╔╝   ██║   ██║
██╔═██╗   ██║   ██║  ██║╚██╗██║    ╚██╔╝    ██║   ██║   
██║  ██╗  ╚██████╔╝  ██║ ╚████║     ██║     ╚██████╔╝   -V {version} Beta  
╚═╝  ╚═╝   ╚═════╝   ╚═╝  ╚═══╝     ╚═╝      ╚═════╝    
                                                
                                                                         
GitHub:{url}

kunyu is Cyberspace Search Engine auxiliary tools

{{datil}}
""".format(version=__version__, url=__url__)

__help__ = """
 _  __                       
| |/ /   _ _ __  _   _ _   _ 
| ' / | | | '_ \| | | | | | |
| . \ |_| | | | | |_| | |_| |
|_|\_\__,_|_| |_|\__, |\__,_| -v{version}
                 |___/         
GitHub: https://github.com/knownsec/Kunyu/
kunyu is Cyberspace Search Engine auxiliary tools
{{datil}}
""".format(version=__version__)

usage = """
Usage:
    kunyu -h
    kunyu console
""".format(module="ZoomEye")

init = """
Usage:
    kunyu -h
    kunyu init -h
    kunyu init --output /root/kunyu/output
    kunyu init --apikey "xxxxx911-6D2A-12345-4e23-64exxxxx6fb"
    kunyu init --username "404@knownsec.com" --password "P@ssword"
    kunyu init --seebug "012345200157abcdef981bcc89a1452c34d62b8c"
    kunyu init --apikey "01234567-acbd-0000" --seebug "a73503200157" (recommend)
    kunyu init --serverless "https://service-xxxxx-xxxxxxx.sh.apigw.tencentcs.com:443"
"""



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: console.py
@Time: 2021/7/21 17:19
'''

import os
import sys

module_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.insert(1, module_path)

from kunyu.config.__version__ import __help__, init
from kunyu.core.console import KunyuInterpreter
from kunyu.utils.log import logger_console
from kunyu.core import conf


def main():
    try:
        # Check whether Kunyu has been initialized, If not, exit the program
        if str(conf.get("zoomeye", "apikey")) == "None" and str(conf.get("login", "token")) == "None":
            raise Exception

        # Call the main class
        KunyuInterpreter().main()
    except Exception:
        # Output initialization help document
        logger_console.info(__help__.format(datil=init))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# encoding: utf-8
"""
@author: 风起
@contact: onlyzaliks@gmail.com
@File: zoomeye.py
@Time: 2021/6/24 22:18
"""

import os
import sys
import json
import random
import datetime

import requests
import platform
from rich.table import Table
from rich.console import Console

try:
    import pocsuite3
except ImportError:
    sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), os.path.pardir)))
from pocsuite3.cli import check_environment, module_path
from pocsuite3 import set_paths
from pocsuite3.lib.core.interpreter import PocsuiteInterpreter
from pocsuite3.lib.core.option import init_options

import kunyu.lib.encode as encode
from kunyu.config.setting import UA, USER_INFO_API, HOST_SEARCH_API, WEB_SEARCH_API, DOMAIN_SEARCH_API, HOST_SCAN_INFO, \
    SEMSITIVE_INFO
from kunyu.core.createmap import create_data_map
from kunyu.lib.export import export_xls
from kunyu.lib.batchfile import get_file
from kunyu.core.crash import HostScan
from kunyu.core.keyword import SearchKeyWord
from kunyu.core import conf
from kunyu.core.seebug import Seebug
from kunyu.utils.log import logger, logger_console
from kunyu.config.__version__ import __help__, init

console = Console(color_system="auto", record=True)
overflow = "fold" if platform.system() == "Darwin" else "ignore"

ZOOMEYE_KEY = conf.get("zoomeye", "apikey")
ZOOMEYE_TOKEN = conf.get("login", "token")

params = {}


class ZoomeyeSearch(object):
    def __init__(self, method):
        self.auth = None
        self.search = None
        self.stype = None
        self.page = 1
        self.method = method
        self.headers = {
            "User-Agent": random.choice(UA),
            "author": "ZoomEye KunYu"
        }

    def __call__(self, func):
        def wrapper(*args, **kwargs):
            nonlocal func
            req_list = []
            login_url = func(self, *args, **kwargs)
            params["sub_type"] = self.stype
            try:
                for num in range(int(self.page)):
                    params['query'], params['page'] = self.search, (num + 1)
                    req_list.append(self.__request(login_url, data=params, headers=self.headers))
            except requests.HTTPError as err:
                logger.warning(err)
            except requests.exceptions.ConnectionError:
                logger.error("Network timeout")
            except KeyboardInterrupt:
                return
            return req_list

        return wrapper

    def __request(self, login_url, data=None, headers=None):
        self.__get_login()
        """"As the request layer,
            the processed data is received and returned to the original data,
            which is displayed on the terminal after processing by the presentation layer.
        """
        time = int(GlobalVar.get_timeout_resp())
        # The API is not available for tourist users
        if self.method == "GET":
            resp = requests.get(
                login_url,
                data=data,
                headers=headers,
                timeout=time,
                verify=False
            )
        else:
            resp = requests.post(
                login_url,
                data=data,
                headers=headers,
                timeout=time,
                verify=False
            )
        self.check_status(resp)
        self.check_error(resp.json())
        # return query data
        return json.loads(resp.text)

    # Check return http status code
    def check_status(self, resp):
        # check http status code 500 or 503
        if resp.status_code in [500, 503]:
            raise requests.HTTPError("ZoomEye Server Error, Status: {}".format(resp.status_code))
        elif resp.status_code in [401]:
            logger.error("The token has expired, please re-initialize")
            print(__help__.format(datil=init))
            sys.exit(0)

    # Check return error info
    def check_error(self, resp):
        if resp.get("error"):
            raise requests.HTTPError(resp.get("message"))

    def __get_login(self):
        """"Obtain the user login credentials and use them dynamically.
            It is recommended to use the API-KEY method to log in,
            because the user name/password requires an additional HTTP request,
            so in theory the API-KEY method is more efficient.
        """
        if ZOOMEYE_KEY == "None":
            self.headers["Authorization"] = "JWT %s" % ZOOMEYE_TOKEN
        else:
            self.headers["API-KEY"] = ZOOMEYE_KEY


# After the SDK public,The interface to get the data.
@ZoomeyeSearch(method="GET")
def _dork_search(self, url, search, page, sub_type):
    """"The logic layer of ZoomEye processes the requested data
        and feeds it back to the request layer to obtain the original data
    """
    try:
        if int(page) <= 0 or page is None:
            raise ArithmeticError
        self.page = page
        self.search = search
        self.stype = sub_type.lower()
        return url

    except ArithmeticError:
        return logger.warning("Please enter the correct number of queries!")
    except Exception:
        return logger.warning("Search for parameter exceptions!")


@ZoomeyeSearch(method="GET")
# Get ZoomEye User Info
def _user_info(self):
    return USER_INFO_API


# Set Global variate
class GlobalVar:
    timeout_resp = 30

    def set_timeout_resp(timeout_resp):
        GlobalVar.timeout_resp = timeout_resp

    def get_timeout_resp(*args):
        return GlobalVar.timeout_resp


# The Display class of the tool
class ZoomEye:
    from kunyu.config.setting import ZOOMEYE_FIELDS_HOST, ZOOMEYE_FIELDS_WEB, ZOOMEYE_FIELDS_INFO, ZOOMEYE_FIELDS_DOMAIN
    from kunyu.utils.convert import convert
    ssl_data_params, raw_data_params, sensitive_params, scatter_params = {}, {}, [], []
    page, dtype, timeout = 1, 0, 30
    stype, btype = "v4", "host"

    # Global commands List
    help = """Global commands:
        info                                      Print User info
        SearchHost <query>                        Basic Host search
        SearchWeb <query>                         Basic Web search
        SearchIcon <File>/<URL>                   Icon Image search
        SearchBatch <File>                        Batch search Host
        SearchCert <Domain>                       SSL certificate Search
        SearchDomain <Domain>                     Domain name associated/subdomain search
        EncodeHash <encryption> <query>           Encryption method interface (base64/hex/md5/mmh3)
        HostCrash <IP> <Domain>                   Host Header Scan hidden assets
        Seebug <query>                            Search Seebug vulnerability information
        set <option>                              Set Global arguments values
        view/views <ID>                           Look over banner row data information
        SearchKeyWord                             Query sensitive information by keyword
        Pocsuite3                                 Invoke the pocsuite component
        ExportPath                                Returns the path of the output file
        CreateMap                                 Generate an IP distribution heat map
        clear                                     Clear the console screen
        show                                      Show can set options
        help                                      Print Help info
        exit                                      Exit KunYu & """

    # ZoomEye Command List
    Command_Info = ["help", "info", "set", "Seebug", "SearchWeb", "SearchHost", "SearchIcon", "HostCrash",
                    "SearchBatch", "SearchCert", "SearchDomain", "EncodeHash", "Pocsuite3", "ExportPath",
                    "show", "clear", "view", "DirectoryCrash", "views", "SearchKeyWord", "CreateMap", "exit"]

    def __init__(self):
        self.fields_tables = None

    def __params_clear(self):
        self.raw_data_params.clear()
        self.ssl_data_params.clear()
        self.sensitive_params.clear()
        self.scatter_params.clear()

    def __command_search(self, search, types="host"):
        """"The raw data obtained is processed and finally displayed on the terminal,
            which is also one of the most core codes
            :param search: zoomeye grammar search statement
            :param types: Dynamically set according to the interface used
        """
        self.__params_clear(self)
        GlobalVar.set_timeout_resp(self.timeout)
        table = Table(show_header=True, style="bold")
        global total, api_url, result, FIELDS, export_host
        result_type = "matches"
        total, num = 0, 0
        export_list = []

        # Gets the API for the call
        api_url, FIELDS = HOST_SEARCH_API, self.ZOOMEYE_FIELDS_HOST
        if types == "web":
            api_url, FIELDS = WEB_SEARCH_API, self.ZOOMEYE_FIELDS_WEB
        elif types == "domain":
            result_type = "list"
            params['q'], params['type'] = search, self.dtype
            api_url, FIELDS = DOMAIN_SEARCH_API, self.ZOOMEYE_FIELDS_DOMAIN

        try:
            for cloumn in FIELDS:
                table.add_column(
                    cloumn, justify="center", overflow=overflow
                )
        except Exception:
            return logger.warning("Please enter the correct field")

        # Get data information
        for result in _dork_search(api_url, search, self.page, self.stype):
            try:
                total = result['total']
                webapp_name, server_name, db_name, system_os, language = "", "", "", "", ""
                for i in range(len(result[result_type])):
                    num += 1
                    title, data_isp = "", ""
                    data = self.convert(result[result_type][i])
                    if api_url == HOST_SEARCH_API:
                        if data.portinfo.title:
                            title = data.portinfo.title[0]
                        try:
                            data_isp = data.geoinfo.isp
                        except:
                            pass

                        # Set the Latitude and longitude information
                        if data.geoinfo.location:
                            lat = data.geoinfo.location.lat
                            lon = data.geoinfo.location.lon

                        # Set the output field
                        table.add_row(str(num), data.ip, str(data.portinfo.port), str(data.portinfo.service),
                                      str(data.portinfo.app), str(data_isp), str(data.geoinfo.country.names.en),
                                      str(data.geoinfo.city.names.en), str(title), str(data.timestamp).split("T")[0])

                        # Set the exported fields
                        export_host = [str(num), data.ip, str(data.portinfo.port), str(data.portinfo.service),
                                       str(data.portinfo.app), str(data_isp), str(data.geoinfo.country.names.en),
                                       str(data.geoinfo.city.names.en), str(title), str(data.timestamp).split("T")[0]]

                        # Set scatter_params info
                        self.scatter_params.append({"lng": str(lon), "lat": str(lat), "ip": data.ip})

                        # Reset the <raw Data Params> element
                        self.raw_data_params[num] = data.portinfo.banner

                        # Reset the <ssl raw Data Params> element
                        try:
                            ssl_raw_data = data.ssl
                            self.ssl_data_params[num] = ssl_raw_data
                        except:
                            pass

                        # Get the sensitive information in the banner
                        sensitive = SearchKeyWord().get_keyword_sensitive(data.portinfo.banner)
                        if sensitive: self.sensitive_params.append(sensitive)

                    elif api_url == WEB_SEARCH_API:
                        # Because of the problem of returning the default value of the field
                        if data.webapp:
                            webapp = self.convert(data.webapp[0])
                            webapp_name = webapp.name
                        if data.server:
                            server = self.convert(data.server[0])
                            server_name = server.name
                        if data.db:
                            db = self.convert(data.db[0])
                            db_name = db.name
                        if data.language:
                            language = data.language[0]
                        if data.system:
                            system = self.convert(data.system[0])
                            system_os = system.name

                        # Set the output field
                        table.add_row(str(num), data.ip[0], str(data.site), str(data.title),
                                      str(system_os), str(webapp_name), str(db_name),
                                      str(language), str(server_name), str(data.timestamp).split("T")[0])

                        # Set the exported fields
                        export_host = [str(num), data.ip[0], str(data.site), str(data.title),
                                       str(system_os), str(webapp_name), str(db_name),
                                       str(language), str(server_name), str(data.timestamp).split("T")[0]]

                        # Reset the <raw Data Params> element
                        self.raw_data_params[num] = data.raw_data

                        # Reset the <ssl raw Data Params> element
                        try:
                            ssl_raw_data = data.ssl
                            self.ssl_data_params[num] = ssl_raw_data
                        except:
                            pass

                        # Get the sensitive information in the banner
                        sensitive = SearchKeyWord().get_keyword_sensitive(data.raw_data)
                        if sensitive: self.sensitive_params.append(sensitive)

                    elif types == "domain":
                        # Set the output field
                        table.add_row(
                            str(num), str(data.name), str(data.ip), str(data.timestamp)
                        )
                        # Set the exported fields
                        export_host = [str(num), str(data.name), str(data.ip), str(data.timestamp)]
                    export_list.append(export_host)

                if export_list:
                    export_xls(export_list, FIELDS)
            except Exception:
                continue

        # Check if the result set is empty
        if total > 0:
            console.log("search result amount:", total, style="green")
            console.print(table)
            logger.info("Search information retrieval is completed\n")
        else:
            logger.error("The query result is empty\n")
        # console.print(self.raw_data_params)
        return console

    @classmethod
    def command_info(cls, *args):
        """ return user info"""
        table = Table(show_header=True, style="bold")
        info = cls.convert(_user_info()[0])
        # Calculate the total quota for the month
        search_quota = int(info.quota_info.remain_free_quota) + int(info.quota_info.remain_pay_quota)
        for column in cls.ZOOMEYE_FIELDS_INFO:
            table.add_column(
                column, justify="center", overflow=overflow
            )
        info.user_info.expired_at = None if info.user_info.expired_at == "" else info.user_info.expired_at
        console.log("User Information:", style="green")
        table.add_row(str(info.user_info.name), str(info.user_info.role),
                      str(search_quota), str(info.user_info.expired_at))
        console.print(table)
        logger.info("User information retrieval is completed\n")

    @classmethod
    # ZoomEye host search method
    def command_searchhost(cls, search):
        return cls.__command_search(cls, search)

    @classmethod
    # ZoomEye web search method
    def command_searchweb(cls, search):
        return cls.__command_search(cls, search, types="web")

    @classmethod
    # domain name associated / subdomain Search
    def command_searchdomain(cls, search):
        return cls.__command_search(cls, search, types="domain")

    @classmethod
    # ZoomEye batch search IP method
    def command_searchbatch(cls, filename):
        """
            Batch query related result sets by reading the IP address in the file
            File name only supports txt type
            :param filename: Accept the IP file path as a parameter
        """
        search = ""
        # Use ZooEye batch query mode,Search: "ip:1.1.1.1 ip:2.2.2.2 ip:3.3.3.3"
        for ip in get_file(filename):
            search += "ip:{} ".format(ip)
        # Determine the type of interface used
        if cls.btype == "host":
            return cls.command_searchhost(search)

        return cls.command_searchweb(search)

    @classmethod
    # ZoomEye SSL Cert Search
    def command_searchcert(cls, hostname):
        """
            Associate assets by calculating the SSL serial number of the domain name
            :param hostname: Enter the domain name using the HTTPS protocol
        """
        try:
            if encode.cert_encode(hostname) is not None:
                return cls.__command_search(cls, "ssl:" + str(encode.cert_encode(hostname)))
        except KeyboardInterrupt:
            return

    @classmethod
    # ZoomEye Icon Image Search
    def command_searchicon(cls, filename):
        """
            Calculate the hash through the icon icon to associate related assets
            According to the input parameters, judge and calculate the ico icon hash of the url or file path
            :param filename: Enter the specified URL or Icon file path
        """
        icon_hash = str(encode.encode_mmh3(filename))
        if icon_hash != "":
            logger.info("iconhash:" + icon_hash)
            return cls.command_searchhost("iconhash:" + icon_hash)

    @classmethod
    # Encode hex/md5/mmh3/base64 Hash
    def command_encodehash(cls, args):
        """
            Characters can be encrypted by this command
            Encryption parameters according to the specified encryption method
            :param args: Obtain the encryption method and the parameters that need to be encrypted
        """
        try:
            command, _, args = args.strip().partition(" ")
            # Check whether a parameter exists
            if args == "":
                raise ArithmeticError
            command_handler = getattr(encode, "encode_{}".format(command))
            # Return the encrypted result
            logger.info("{} : {}".format(
                command,
                command_handler(args.strip())
            ))

        except ArithmeticError:
            return logger.warning("Please specify the encryption mode")

        except Exception as err:
            return logger.warning(err)

    @classmethod
    # Get SeeBug vulnerability information
    def command_seebug(cls, search):
        """
            Get the historical vulnerabilities of the specified framework
            You can use this command to query the historical vulnerabilities of related frameworks
            :param search: Used to obtain parameters for querying related framework vulnerabilities
        """
        # Get Number of result sets of vulnerabilities
        total = Seebug.search(search).get("total")
        # Get vulnerabilities result
        data = Seebug.search(search)
        logger.info("Number of relevant vulnerabilities: {}".format(total))
        for vuln in data["results"]:
            vuln = cls.convert(vuln)
            # Print Result set of framework history vulnerabilities
            logger_console.info('[{}] - [{}]'.format(vuln.name, vuln.id))

        logger.info("Seebug Search retrieval is completed\n")

    @classmethod
    # Invoke the pocsuite component
    def command_pocsuite3(cls, *args, **kwargs):
        check_environment()
        set_paths(module_path())
        init_options()
        # Set the directory to store the poc
        poc = PocsuiteInterpreter(os.path.abspath(os.path.dirname(os.path.dirname(__file__))) + "/pocs")
        poc.start()

    @classmethod
    # look over row_data info
    def command_view(cls, serial):
        """
            View raw data information
            You can view any raw data by entering the serial number
            :param serial): Please enter serial number ID
        """
        try:
            # If the key parameter is not specified, the key parameter is automatically set to 1
            serials = 1 if serial is "" else serial
            raw_data = cls.raw_data_params.get(int(serials))
            # Check whether the returned result is None
            if raw_data is None:
                raise ArithmeticError
            # Check whether the returned result is ""
            elif raw_data is "":
                logger.warning("Banner information is empty")
            else:
                console.log("Banner Information is:\n", style="green")
                console.print(raw_data)

        except ArithmeticError:
            return logger.warning(
                "No retrieval operation is performed or the length of the dictionary key value is exceeded")

        finally:
            print()

    @classmethod
    # look over ssl row_data info
    def command_views(cls, serial):
        """
            View ssl raw data information
            You can views any ssl raw data by entering the serial number
            :param serial): Please enter serial number ID
        """
        try:
            # If the key parameter is not specified, the key parameter is automatically set to 1
            serials = 1 if serial is "" else serial
            ssl_raw_data = cls.ssl_data_params.get(int(serials))
            # Check whether the returned result is None
            if ssl_raw_data is None:
                raise ArithmeticError
            else:
                console.log("SSL Banner Information is:\n", style="green")
                console.print(ssl_raw_data)
        except ArithmeticError:
            return logger.warning("SSL Banner information is empty")

        finally:
            print()

    @classmethod
    def command_searchkeyword(cls, *args, **kwargs):
        try:
            keyword_param = set()
            num = 0
            # sensitive_params information
            for sensitive in cls.sensitive_params:
                for item in sensitive:
                    keyword_param.add(item)
            # Set table console print
            table = Table(show_header=True, style="bold")
            for column in SEMSITIVE_INFO:
                table.add_column(
                    column, justify="center", overflow=overflow
                )
            # add table information
            for keyword in keyword_param:
                num += 1
                table.add_row(
                    str(num), str(keyword)
                )
            # print sensitive info
            console.print(table)
        except Exception:
            return

    @classmethod
    def command_createmap(cls, *args, **kwargs):
        """
            Create a network space asset distribution map
            You can use the CreateMap command to generate a resource distribution map after a search
        """
        # Check whether Scatter Params is empty
        if len(cls.scatter_params) == 0: return logger.warning("Asset List is Empty,Please search and try again")
        from kunyu.config import setting
        # Generating an export path
        __filename = datetime.datetime.now().strftime("ScatterGram_%H%M%S.html")
        path = os.path.join(setting.OUTPUT_PATH, __filename)
        create_data_map(cls.scatter_params, path)
        # Output Export path
        logger.info(path)

    @classmethod
    def command_hostcrash(cls, args):
        """
            HOST Header Crash function,Obtaining hidden Assets
            You can use the master domain name reverse lookup or file to obtain the domain name list
            :param args: The IP address and domain name to be queried
        """
        try:
            # Check whether a parameter exists
            if args == "":
                raise ArithmeticError
            # Get args ip and search
            ip, _, search = args.strip().partition(" ")
            result_list = HostScan().host_scan(search, ip)
            # Determines whether the result set is empty
            if not result_list:
                return logger.warning("The query result is empty\n")
            table = Table(show_header=True, style="bold")
            # Read the list header
            for column in HOST_SCAN_INFO:
                table.add_column(column, justify="center", overflow=overflow)
            # Set output list content
            for res in result_list:
                table.add_row(
                    str(res[0]), str(res[1]),str(res[2])
                )
            # Output table list to console
            console.print(table)
            logger.info("Host Header Scan is completed\n")
            # Export HOSTS collision results
            export_xls(result_list, HOST_SCAN_INFO)
            # End of function execution
        except ArithmeticError:
            return logger.warning("Please Host IP and Domain")

        except KeyboardInterrupt:
            return



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: setup.py
@Time: 2021/7/24
'''


import os
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

    def find_packages(where='.'):
        # os.walk -> list[(dirname, list[subdirs], list[files])]
        return [folder.replace("/", ".").lstrip(".")
                for (folder, _, fils) in os.walk(where)
                if "__init__.py" in fils]

from kunyu.config.__version__ import __version__


DEPENDENCIES = open('requirements.txt', 'r', encoding='utf-8').read().split('\n')
README = open('README.md', 'r', encoding='utf-8').read()

setup(
    name='kunyu',
    version=__version__,
    description='kunyu is Cyberspace Resources Surveying and Mapping auxiliary tools',
    long_description=README,
    long_description_content_type='text/markdown',
    author='风起 <onlyzaliks@gmail.com>',
    url='',
    packages=find_packages(),
    include_package_data=True,
    entry_points={'console_scripts': ['kunyu=kunyu.console:main']},
    install_requires=DEPENDENCIES,
    keywords=['Cyberspace Resources', 'zoomeye', 'kunyu'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: console.py
@Time: 2021/7/19 22:35
'''

import os
import sys
import platform
import subprocess

from colorama import init
from rich.table import Table
from rich.console import Console

from kunyu.config import setting
from kunyu.utils.log import logger
from kunyu.lib.export import createdir
from kunyu.core.zoomeye import ZoomEye
from kunyu.utils import readineng as readline
from kunyu.config.__version__ import __introduction__
from kunyu.config.setting import COMMAND_INFO, OS_SYSTEM

init(autoreset=True)
PLATFORM = platform.system()

# Determine the operating system clear screen command
cmd = "cls" if PLATFORM == "Windows" else "clear"
console = Console(color_system="auto", record=True)


def readline_available():
    """
    Check if the readline is available. By default
    it is not in Python default installation on Windows
    """
    return readline._readline is not None


# TAB auto completion
def auto_completion(completion=None, console=None):
    if not readline_available():
        return
    readline.set_completer_delims(" ")
    readline.set_completer(console)
    readline.parse_and_bind("tab: complete")


class BaseInterpreter(object):
    global_help = ""
    OUTPUT_PATH = None

    def __init__(self):
        self.module = "ZoomEye"
        self.complet = eval(self.module).Command_Info
        self.setup()
        # Create output directory
        createdir()

    def setup(self):
        """ Initialization of third-party libraries
        Setting appropriate completer function.
        """
        auto_completion(completion=4, console=self.complete)

    def parse_line(self, line):
        command, _, args = line.strip().partition(" ")
        return command, args.strip()

    def setter(self, line):
        """"
        :line set options
        """
        args, _, number = line.strip().partition("=")
        args, number = args.strip(), number.strip()
        if hasattr(eval(self.module), args):
            setattr(eval(self.module), args, number)

        return True

    def getter(self, line):
        """"
        :return options value
        """
        return getattr(eval(self.module), line)

    @property
    def prompt(self):
        """ Returns prompt string """
        os.system("")
        return "Kunyu (\033[31;1m{moudle}\033[0m) > ".format(moudle=self.module)

    def get_command_handler(self, command):
        """ Parsing command and returning appropriate handler.
        :param command: command
        :return: command_handler
        """
        try:
            command_handler = getattr(eval(self.module), "command_{}".format(command))

        except AttributeError:
            logger.error("Unknown command: '{}'".format(command))
            return False

        return command_handler

    def complete(self, text, state):
        """Return the next possible completion for 'text'.
        If a command has not been entered, then complete against command list.
        Otherwise try to call complete_<command> to get list of completions.
        """
        if state == 0:
            start_index = readline.get_begidx()
            if start_index != 0:
                complete_function = self.default_completer
            else:
                complete_function = self.raw_command_completer

            self.completion_matches = complete_function(text)
        try:
            return self.completion_matches[state]
        except IndexError:
            return None

    def raw_command_completer(self, text):
        """ Complete command w/o any argument """
        return [command for command in self.complet if command.lower().startswith(text.lower())]

    def default_completer(self, *ignored):
        return []

    def auxiliary(self, command, line=None):
        """"Set how to handle basic commands
        :return True/False
        """
        if command == "clear":
            subprocess.call(cmd, shell=True)
            return True
        elif command == "exit":
            raise KeyboardInterrupt

        elif command == "help":
            console.print(eval(self.module).help)
            return True
        elif command == "set":
            return self.setter(line)

        # show Global Command Info
        elif command == "show":
            table = Table(show_header=True, style="bold")
            for cloumn in COMMAND_INFO:
                table.add_column(
                    cloumn, justify="center", overflow="fold"
                )
            command_info = [["page", self.getter("page"),"Set Search Page"],
                            ["dtype", self.getter("dtype"), "Set Associated/Subdomain Search Schema"],
                            ["stype", self.getter("stype"), "Set data type IPV4/IPV6 (option v4/v6)"],
                            ["btype", self.getter("btype"), "Set BatchFile Search Schema"],
                            ["timeout", self.getter("timeout"), "Set HTTP Requests Timeout"]]
            for info in command_info:
                table.add_row(
                    str(info[0]), str(info[1]), str(info[2])
                )
            console.log("Global Command Info:", style="green")
            console.print(table)
            return True
        elif command == "exportpath":
            # Return Export Path
            logger.info(setting.OUTPUT_PATH)
            return True

        elif command in OS_SYSTEM:
            try:
                command_os = "{} {}".format(command, line)
                os.system(command_os)
            except KeyboardInterrupt:
                print("")
            return True

        return False

    def start(self):
        # logger_console(self.global_help)
        while True:
            self.setup()
            try:
                command, args = self.parse_line(input(self.prompt))
                command = command.lower()
                if self.auxiliary(command, args) or not command:
                    continue
                command_handler = self.get_command_handler(command)
                command_handler(args)

            except EOFError:
                logger.info("kunyu Console mode stopped")
                break
            except KeyboardInterrupt:
                logger.info("kunyu Console mode stopped")
                try:
                    os.rmdir(setting.OUTPUT_PATH)
                except OSError:
                    pass
                sys.exit(0)

            except Exception as err:
                console.print(err)
                continue


class KunyuInterpreter(BaseInterpreter):

    def __init__(self):
        super().__init__()
        self.global_help = __introduction__.format(datil=eval(self.module).help)
        console.print(self.global_help)

    def main(self):
        super().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: batchfile.py
@Time: 2021/6/16 21:05
'''

import sys
import re

from functools import wraps

from kunyu.config.setting import IP_ADDRESS_REGEX, DOMAIN_CHECK_REGEX
from kunyu.utils.log import logger


def getresult(func):


    @wraps(func)
    def getfile(file):
        """
            Read the content in the txt file that meets the requirements of the processing layer
            Determine whether it is a txt file
            :param file: The path of the file to be read
        """
        __ip_list = []
        try:
            # Check File Type
            if file.endswith(".txt"):
                logger.info("File load successful")
            else:
                logger.warning("Only input TXT type files are allowed")
                raise Exception

            with open(file, "r", encoding='utf-8') as ip_text:
                for line in ip_text:
                    __ip_list.append(line.strip())
            return filter(func, __ip_list)
        except Exception:
            # ([LRE])Hidden garbled code appearing before the file path can also cause an error.
            # Copy from right to left will lead to appear [LRE] tabs,Whereas not.
            return logger.error("please check the file name is correct")

    return getfile


@getresult
def __check_file_ip(ip):
    # Check IP legitimacy
    return True if re.search(IP_ADDRESS_REGEX, ip) else logger.warning(ip + ":It's an illegal IP address")


@getresult
def __check_file_domain(domain):
    # Check DOMAIN legitimacy
    return True if re.search(DOMAIN_CHECK_REGEX, domain) else logger.warning(domain + ":It's an illegal Domain")


# Get the contents of the Domain list.
def get_domain_file(*args, **kwargs):
    _ip_list = []
    try:
        for i in __check_file_domain(*args, **kwargs):
            _ip_list.append(i)
        return _ip_list

    except KeyboardInterrupt:
        return
    except Exception:
        logger.error("Failed to get IP list content! Please check if the IP file name is abnormal")


# Get the contents of the IP list.
def get_file(*args, **kwargs):
    _ip_list = []
    try:
        for i in __check_file_ip(*args, **kwargs):
            _ip_list.append(i)
        return _ip_list

    except KeyboardInterrupt:
        return
    except Exception:
        logger.error("Failed to get IP list content! Please check if the IP file name is abnormal")



#!/usr/bin/env python
# encoding: utf-8
'''
@author: 风起
@contact: onlyzaliks@gmail.com
@File: seebug.py
@Time: 2021/7/20 16:42
'''
import json
from urllib.parse import urlencode

import requests

from kunyu.core import conf
from kunyu.utils.log import logger


SEARCH_API = "https://www.seebug.org/api/get_open_vuls_by_component"
SEARCH_VUL_API = "https://www.seebug.org/api/get_vul_detail_by_id"

SEEBUG_KEY = conf


class Seebug:
    """Obtain component related historical vulnerabilities
       through the Seebug interface
    """
    headers = {
        "Content-Type": "application/json"
    }

    def __init__(self):
        self.__get_login()
        self.param = {}

    # Get Seebug user login
    def __get_login(self):
        if SEEBUG_KEY is not None:
            self.headers["Authorization"] = "Token {}".format(SEEBUG_KEY)

    @classmethod
    def search(cls, search):
        try:
            cls.param = {
                "app_name": search
            }
            login_url = "%s?%s" % (SEARCH_API, urlencode(cls.param))
            resp = requests.get(
                login_url,
                headers=cls.headers,
                verify=False
            )
            return json.loads(resp.text)

        except KeyboardInterrupt:
            return

        except Exception:
            return logger.error("Failed to get SeeBug vulnerability information")










